#ifndef SRC_CONTACTNETWORKFACTORY_HH
#define SRC_CONTACTNETWORKFACTORY_HH

#include <dune/common/parametertree.hh>

#include "../data-structures/contactnetwork.hh"
#include "../data-structures/levelcontactnetwork.hh"

#include "levelcontactnetworkfactory.hh"

template <class GridType, int dim>
class ContactNetworkFactory {
public:
    using ContactNetwork = ContactNetwork<GridType, dim>;

protected:
    using LevelContactNetworkFactory = LevelContactNetworkFactory<GridType, dim>;

    const Dune::ParameterTree& parset_;

    std::vector<std::shared_ptr<LevelContactNetworkFactory>> levelContactNetworkFactories_;

    ContactNetwork contactNetwork_;

private:
    virtual void setBodies() = 0;
    virtual void setCouplings() = 0;
    virtual void setBoundaryConditions() = 0;

public:
    ContactNetworkFactory(const Dune::ParameterTree& parset) :
        parset_(parset),
        contactNetwork_() {}

    void build() {
        setBodies();
        contactNetwork_.setBodies();

        setCouplings();
        contactNetwork_.setCouplings();

        setBoundaryConditions();
    }

    ContactNetwork& levelContactNetwork() {
        return contactNetwork_;
    }
};
#endif
