#ifndef SRC_MULTI_BODY_PROBLEM_DATA_CUBOIDGEOMETRY_HH
#define SRC_MULTI_BODY_PROBLEM_DATA_CUBOIDGEOMETRY_HH

#include <dune/common/fvector.hh>

#include "../midpoint.hh"

class CuboidGeometry {
  public:
    typedef Dune::FieldVector<double, MY_DIM> LocalVector;

    constexpr static double const lengthScale = 1.0; // scaling factor

  private:
    const double length_;
    const double width_;
   // const double lowerWeakLength_;     // length of the lower weak zone
   // const double upperWeakLength_;     // length of the lower weak zone

  public:
    // corners of cube
    const LocalVector A;
    const LocalVector B;
    const LocalVector C;
    const LocalVector D;

    // corners of lower weakening patch
    const LocalVector V;
    const LocalVector W;

    // corners of upper weakening patch
    const LocalVector X;
    const LocalVector Y;

#if MY_DIM == 3
    const double depth;

    CuboidGeometry(const LocalVector& origin,
                   const LocalVector& lowerWeakOrigin, const LocalVector& upperWeakOrigin,
                   const double lowerWeakLength = 0.20, const double upperWeakLength = 0.20,
                   const double length = 1.00, const double width = 0.27, const double depth = 0.60);
#else
        CuboidGeometry(const LocalVector& origin,
                       const LocalVector& lowerWeakOrigin, const LocalVector& upperWeakOrigin,
                       const double lowerWeakLength = 0.20, const double upperWeakLength = 0.20,
                       const double length = 1.00, const double width = 0.27);
#endif

    void write() const;

   // void render() const;
};
#endif
