#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dune/solvers/common/wrapownshare.hh>
#include <dune/solvers/iterationsteps/blockgssteps.hh>
#include <dune/solvers/solvers/umfpacksolver.hh>

#include "solverfactory.hh"

#include "../utils/debugutils.hh"

template <class Functional, class BitVector>
SolverFactory<Functional, BitVector>::SolverFactory(
    const Dune::ParameterTree& parset,
    Functional& J,
    const BitVector& ignoreNodes) :
      J_(Dune::Solvers::wrap_own_share<const Functional>(std::forward<Functional>(J)))
      //linear solver
      //preconditioner_(),
      //linearSolverStep_(),
    {
    nonlinearSmoother_ = std::make_shared<NonlinearSmoother>(*J_, dummyIterate_, LocalSolver());

    // linearSolverStep_.setPreconditioner(preconditioner_);
    linearSolverStep_ = Dune::Solvers::BlockGSStepFactory<Matrix, Vector>::createPtr(Dune::Solvers::BlockGS::LocalSolvers::ldlt());
    energyNorm_ = std::make_shared<EnergyNorm<Matrix, Vector>>(*linearSolverStep_);
    //linearSolver_ = std::make_shared<LinearSolver>(*linearSolverStep_, parset.get<size_t>("linear.maximumIterations"), parset.get<double>("linear.tolerance"), *energyNorm_, Solver::QUIET);
    linearSolver_ = std::make_shared<Dune::Solvers::UMFPackSolver<Matrix, Vector>>();

    tnnmgStep_ = std::make_shared<Step>(*J_, dummyIterate_, nonlinearSmoother_, linearSolver_, DefectProjection(), LineSearchSolver());
    tnnmgStep_->setPreSmoothingSteps(parset.get<int>("main.pre"));
    tnnmgStep_->setIgnore(ignoreNodes);
}

template <class Functional, class BitVector>
void SolverFactory<Functional, BitVector>::setProblem(Vector& x) {
    nonlinearSmoother_->setProblem(x);
    tnnmgStep_->setProblem(x);
}


template <class Functional, class BitVector>
auto SolverFactory<Functional, BitVector>::step()
-> std::shared_ptr<Step> {
    return tnnmgStep_;
}

#include "solverfactory_tmpl.cc"
